@@ -15,7 +15,6 @@
  */
 
 #define C_CFISH_HASH
-#define C_CFISH_HASHTOMBSTONE
 #define CFISH_USE_SHORT_NAMES
 
 #include <string.h>
@@ -26,26 +25,24 @@
 #include "Clownfish/Hash.h"
 #include "Clownfish/String.h"
 #include "Clownfish/Err.h"
-#include "Clownfish/VArray.h"
+#include "Clownfish/Vector.h"
 #include "Clownfish/Util/Memory.h"
 
-static HashTombStone *TOMBSTONE;
+// TOMBSTONE is shared across threads, so it must never be incref'd or
+// decref'd.
+static String *TOMBSTONE;
 
 #define HashEntry cfish_HashEntry
 
 typedef struct HashEntry {
-    Obj     *key;
-    Obj     *value;
-    int32_t  hash_sum;
+    String *key;
+    Obj    *value;
+    size_t  hash_sum;
 } HashEntry;
 
-// Reset the iterator.  Hash_Iterate must be called to restart iteration.
-static CFISH_INLINE void
-SI_kill_iter(Hash *self);
-
 // Return the entry associated with the key, if any.
 static CFISH_INLINE HashEntry*
-SI_fetch_entry(Hash *self, Obj *key, int32_t hash_sum);
+SI_fetch_entry(Hash *self, String *key, size_t hash_sum);
 
 // Double the number of buckets and redistribute all entries.
 static CFISH_INLINE HashEntry*
@@ -53,36 +50,34 @@ SI_rebuild_hash(Hash *self);
 
 void
 Hash_init_class() {
-    TOMBSTONE = (HashTombStone*)Class_Make_Obj(HASHTOMBSTONE);
+    TOMBSTONE = Str_newf("[HASHTOMBSTONE]");
 }
 
 Hash*
-Hash_new(uint32_t capacity) {
+Hash_new(size_t capacity) {
     Hash *self = (Hash*)Class_Make_Obj(HASH);
     return Hash_init(self, capacity);
 }
 
 Hash*
-Hash_init(Hash *self, uint32_t capacity) {
+Hash_init(Hash *self, size_t min_threshold) {
     // Allocate enough space to hold the requested number of elements without
     // triggering a rebuild.
-    uint32_t requested_capacity = capacity < INT32_MAX ? capacity : INT32_MAX;
-    uint32_t threshold;
-    capacity = 16;
-    while (1) {
+    size_t threshold;
+    size_t capacity = 16;
+    do {
         threshold = (capacity / 3) * 2;
-        if (threshold > requested_capacity) { break; }
+        if (threshold > min_threshold) { break; }
         capacity *= 2;
-    }
+    } while (capacity <= SIZE_MAX / 2);
 
     // Init.
-    self->size         = 0;
-    self->iter_tick    = -1;
+    self->size      = 0;
 
     // Derive.
-    self->capacity     = capacity;
-    self->entries      = (HashEntry*)CALLOCATE(capacity, sizeof(HashEntry));
-    self->threshold    = threshold;
+    self->capacity  = capacity;
+    self->entries   = (HashEntry*)CALLOCATE(capacity, sizeof(HashEntry));
+    self->threshold = threshold;
 
     return self;
 }
@@ -104,6 +99,10 @@ Hash_Clear_IMP(Hash *self) {
     // Iterate through all entries.
     for (; entry < limit; entry++) {
         if (!entry->key) { continue; }
+        if (entry->key == TOMBSTONE) {
+            entry->key = NULL;
+            continue;
+        }
         DECREF(entry->key);
         DECREF(entry->value);
         entry->key       = NULL;
@@ -112,11 +111,13 @@ Hash_Clear_IMP(Hash *self) {
     }
 
     self->size = 0;
+    // All tombstones were removed, reset threshold.
+    self->threshold = (self->capacity / 3) * 2;
 }
 
-void
-Hash_do_store(Hash *self, Obj *key, Obj *value,
-              int32_t hash_sum, bool use_this_key) {
+static void
+S_do_store(Hash *self, String *key, Obj *value, size_t hash_sum,
+           bool incref_key) {
     HashEntry *entry = SI_fetch_entry(self, key, hash_sum);
     if (entry) {
         DECREF(entry->value);
@@ -127,20 +128,20 @@ Hash_do_store(Hash *self, Obj *key, Obj *value,
     HashEntry *entries = self->size >= self->threshold
                          ? SI_rebuild_hash(self)
                          : (HashEntry*)self->entries;
-    uint32_t       tick = hash_sum;
-    const uint32_t mask = self->capacity - 1;
+    size_t       tick = hash_sum;
+    const size_t mask = self->capacity - 1;
 
     while (1) {
         tick &= mask;
         HashEntry *entry = entries + tick;
-        if (entry->key == (Obj*)TOMBSTONE || !entry->key) {
-            if (entry->key == (Obj*)TOMBSTONE) {
+        if (entry->key == TOMBSTONE || !entry->key) {
+            if (entry->key == TOMBSTONE) {
                 // Take note of diminished tombstone clutter.
                 self->threshold++;
             }
-            entry->key       = use_this_key
-                               ? key
-                               : Hash_Make_Key(self, key, hash_sum);
+            entry->key       = incref_key
+                               ? (String*)INCREF(key)
+                               : key;
             entry->value     = value;
             entry->hash_sum  = hash_sum;
             self->size++;
@@ -151,33 +152,25 @@ Hash_do_store(Hash *self, Obj *key, Obj *value,
 }
 
 void
-Hash_Store_IMP(Hash *self, Obj *key, Obj *value) {
-    Hash_do_store(self, key, value, Obj_Hash_Sum(key), false);
+Hash_Store_IMP(Hash *self, String *key, Obj *value) {
+    S_do_store(self, key, value, Str_Hash_Sum(key), true);
 }
 
 void
 Hash_Store_Utf8_IMP(Hash *self, const char *key, size_t key_len, Obj *value) {
-    StackString *key_buf = SSTR_WRAP_UTF8((char*)key, key_len);
-    Hash_do_store(self, (Obj*)key_buf, value,
-                  SStr_Hash_Sum(key_buf), false);
-}
-
-Obj*
-Hash_Make_Key_IMP(Hash *self, Obj *key, int32_t hash_sum) {
-    UNUSED_VAR(self);
-    UNUSED_VAR(hash_sum);
-    return Obj_Clone(key);
+    String *key_buf = SSTR_WRAP_UTF8((char*)key, key_len);
+    S_do_store(self, key_buf, value, Str_Hash_Sum(key_buf), true);
 }
 
 Obj*
 Hash_Fetch_Utf8_IMP(Hash *self, const char *key, size_t key_len) {
-    StackString *key_buf = SSTR_WRAP_UTF8(key, key_len);
-    return Hash_Fetch_IMP(self, (Obj*)key_buf);
+    String *key_buf = SSTR_WRAP_UTF8(key, key_len);
+    return Hash_Fetch_IMP(self, key_buf);
 }
 
 static CFISH_INLINE HashEntry*
-SI_fetch_entry(Hash *self, Obj *key, int32_t hash_sum) {
-    uint32_t tick = hash_sum;
+SI_fetch_entry(Hash *self, String *key, size_t hash_sum) {
+    size_t tick = hash_sum;
     HashEntry *const entries = (HashEntry*)self->entries;
     HashEntry *entry;
 
@@ -189,7 +182,8 @@ SI_fetch_entry(Hash *self, Obj *key, int32_t hash_sum) {
             return NULL;
         }
         else if (entry->hash_sum == hash_sum
-                 && Obj_Equals(key, entry->key)
+                 && entry->key != TOMBSTONE
+                 && Str_Equals(key, (Obj*)entry->key)
                 ) {
             return entry;
         }
@@ -198,18 +192,18 @@ SI_fetch_entry(Hash *self, Obj *key, int32_t hash_sum) {
 }
 
 Obj*
-Hash_Fetch_IMP(Hash *self, Obj *key) {
-    HashEntry *entry = SI_fetch_entry(self, key, Obj_Hash_Sum(key));
+Hash_Fetch_IMP(Hash *self, String *key) {
+    HashEntry *entry = SI_fetch_entry(self, key, Str_Hash_Sum(key));
     return entry ? entry->value : NULL;
 }
 
 Obj*
-Hash_Delete_IMP(Hash *self, Obj *key) {
-    HashEntry *entry = SI_fetch_entry(self, key, Obj_Hash_Sum(key));
+Hash_Delete_IMP(Hash *self, String *key) {
+    HashEntry *entry = SI_fetch_entry(self, key, Str_Hash_Sum(key));
     if (entry) {
         Obj *value = entry->value;
         DECREF(entry->key);
-        entry->key       = (Obj*)TOMBSTONE;
+        entry->key       = TOMBSTONE;
         entry->value     = NULL;
         entry->hash_sum  = 0;
         self->size--;
@@ -223,119 +217,99 @@ Hash_Delete_IMP(Hash *self, Obj *key) {
 
 Obj*
 Hash_Delete_Utf8_IMP(Hash *self, const char *key, size_t key_len) {
-    StackString *key_buf = SSTR_WRAP_UTF8(key, key_len);
-    return Hash_Delete_IMP(self, (Obj*)key_buf);
-}
-
-uint32_t
-Hash_Iterate_IMP(Hash *self) {
-    SI_kill_iter(self);
-    return self->size;
-}
-
-static CFISH_INLINE void
-SI_kill_iter(Hash *self) {
-    self->iter_tick = -1;
+    String *key_buf = SSTR_WRAP_UTF8(key, key_len);
+    return Hash_Delete_IMP(self, key_buf);
 }
 
 bool
-Hash_Next_IMP(Hash *self, Obj **key, Obj **value) {
-    while (1) {
-        if (++self->iter_tick >= (int32_t)self->capacity) {
-            // Bail since we've completed the iteration.
-            --self->iter_tick;
-            *key   = NULL;
-            *value = NULL;
-            return false;
-        }
-        else {
-            HashEntry *const entry
-                = (HashEntry*)self->entries + self->iter_tick;
-            if (entry->key && entry->key != (Obj*)TOMBSTONE) {
-                // Success!
-                *key   = entry->key;
-                *value = entry->value;
-                return true;
-            }
-        }
-    }
+Hash_Has_Key_IMP(Hash *self, String *key) {
+    HashEntry *entry = SI_fetch_entry(self, key, Str_Hash_Sum(key));
+    return entry ? true : false;
 }
 
-Obj*
-Hash_Find_Key_IMP(Hash *self, Obj *key, int32_t hash_sum) {
-    HashEntry *entry = SI_fetch_entry(self, key, hash_sum);
-    return entry ? entry->key : NULL;
-}
-
-VArray*
+Vector*
 Hash_Keys_IMP(Hash *self) {
-    Obj *key;
-    Obj *val;
-    VArray *keys = VA_new(self->size);
-    Hash_Iterate(self);
-    while (Hash_Next(self, &key, &val)) {
-        VA_Push(keys, INCREF(key));
+    Vector    *keys        = Vec_new(self->size);
+    HashEntry *entry       = (HashEntry*)self->entries;
+    HashEntry *const limit = entry + self->capacity;
+
+    for (; entry < limit; entry++) {
+        if (entry->key && entry->key != TOMBSTONE) {
+            Vec_Push(keys, INCREF(entry->key));
+        }
     }
+
     return keys;
 }
 
-VArray*
+Vector*
 Hash_Values_IMP(Hash *self) {
-    Obj *key;
-    Obj *val;
-    VArray *values = VA_new(self->size);
-    Hash_Iterate(self);
-    while (Hash_Next(self, &key, &val)) { VA_Push(values, INCREF(val)); }
+    Vector    *values      = Vec_new(self->size);
+    HashEntry *entry       = (HashEntry*)self->entries;
+    HashEntry *const limit = entry + self->capacity;
+
+    for (; entry < limit; entry++) {
+        if (entry->key && entry->key != TOMBSTONE) {
+            Vec_Push(values, INCREF(entry->value));
+        }
+    }
+
     return values;
 }
 
 bool
 Hash_Equals_IMP(Hash *self, Obj *other) {
     Hash    *twin = (Hash*)other;
-    Obj     *key;
-    Obj     *val;
 
     if (twin == self)             { return true; }
-    if (!Obj_Is_A(other, HASH))   { return false; }
+    if (!Obj_is_a(other, HASH))   { return false; }
     if (self->size != twin->size) { return false; }
 
-    Hash_Iterate(self);
-    while (Hash_Next(self, &key, &val)) {
-        Obj *other_val = Hash_Fetch(twin, key);
-        if (!other_val || !Obj_Equals(other_val, val)) { return false; }
+    HashEntry *entry       = (HashEntry*)self->entries;
+    HashEntry *const limit = entry + self->capacity;
+
+    for (; entry < limit; entry++) {
+        if (entry->key && entry->key != TOMBSTONE) {
+            Obj *other_val = Hash_Fetch(twin, entry->key);
+            if (!other_val || !Obj_Equals(other_val, entry->value)) {
+                return false;
+            }
+        }
     }
 
     return true;
 }
 
-uint32_t
+size_t
 Hash_Get_Capacity_IMP(Hash *self) {
     return self->capacity;
 }
 
-uint32_t
+size_t
 Hash_Get_Size_IMP(Hash *self) {
     return self->size;
 }
 
 static CFISH_INLINE HashEntry*
 SI_rebuild_hash(Hash *self) {
+    if (self->capacity > SIZE_MAX / 2) {
+        THROW(ERR, "Hash grew too large");
+    }
+
     HashEntry *old_entries = (HashEntry*)self->entries;
     HashEntry *entry       = old_entries;
     HashEntry *limit       = old_entries + self->capacity;
 
-    SI_kill_iter(self);
     self->capacity *= 2;
     self->threshold = (self->capacity / 3) * 2;
     self->entries   = (HashEntry*)CALLOCATE(self->capacity, sizeof(HashEntry));
     self->size      = 0;
 
     for (; entry < limit; entry++) {
-        if (!entry->key || entry->key == (Obj*)TOMBSTONE) {
+        if (!entry->key || entry->key == TOMBSTONE) {
             continue;
         }
-        Hash_do_store(self, entry->key, entry->value,
-                      entry->hash_sum, true);
+        S_do_store(self, entry->key, entry->value, entry->hash_sum, false);
     }
 
     FREEMEM(old_entries);
@@ -343,23 +317,8 @@ SI_rebuild_hash(Hash *self) {
     return (HashEntry*)self->entries;
 }
 
-/***************************************************************************/
-
-uint32_t
-HashTombStone_Get_RefCount_IMP(HashTombStone* self) {
-    UNUSED_VAR(self);
-    return 1;
+String*
+Hash_get_tombstone() {
+    return TOMBSTONE;
 }
 
-HashTombStone*
-HashTombStone_Inc_RefCount_IMP(HashTombStone* self) {
-    return self;
-}
-
-uint32_t
-HashTombStone_Dec_RefCount_IMP(HashTombStone* self) {
-    UNUSED_VAR(self);
-    return 1;
-}
-
-
