@@ -14,6 +14,8 @@
  * limitations under the License.
  */
 
+#include <ctype.h>
+
 #define C_CFISH_METHOD
 #define CFISH_USE_SHORT_NAMES
 
@@ -21,6 +23,7 @@
 #include "Clownfish/String.h"
 #include "Clownfish/Err.h"
 #include "Clownfish/Class.h"
+#include "Clownfish/CharBuf.h"
 
 Method*
 Method_new(String *name, cfish_method_t callback_func, uint32_t offset) {
@@ -81,4 +84,29 @@ Method_Is_Excluded_From_Host_IMP(Method *self) {
     return self->is_excluded;
 }
 
+String*
+Method_lower_snake_alias(cfish_Method *method) {
+    cfish_String *host_alias = CFISH_Method_Get_Host_Alias(method);
+    if (host_alias) {
+        return (cfish_String*)CFISH_INCREF(host_alias);
+    }
 
+    // Convert to lowercase.
+    cfish_String *name = CFISH_Method_Get_Name(method);
+    cfish_CharBuf *buf = cfish_CB_new(CFISH_Str_Get_Size(name));
+    cfish_StringIterator *iter = CFISH_Str_Top(name);
+    int32_t code_point;
+    while (CFISH_STR_OOB != (code_point = CFISH_StrIter_Next(iter))) {
+        if (code_point > 127) {
+            THROW(CFISH_ERR, "Can't lowercase '%o'", name);
+        }
+        else {
+            CFISH_CB_Cat_Char(buf, tolower(code_point));
+        }
+    }
+    cfish_String *retval = CFISH_CB_Yield_String(buf);
+    CFISH_DECREF(iter);
+    CFISH_DECREF(buf);
+
+    return retval;
+}
