@@ -23,12 +23,12 @@
 #include "Clownfish/Test/TestHash.h"
 
 #include "Clownfish/String.h"
+#include "Clownfish/Boolean.h"
 #include "Clownfish/Hash.h"
-#include "Clownfish/Num.h"
 #include "Clownfish/Test.h"
 #include "Clownfish/TestHarness/TestBatchRunner.h"
 #include "Clownfish/TestHarness/TestUtils.h"
-#include "Clownfish/VArray.h"
+#include "Clownfish/Vector.h"
 #include "Clownfish/Class.h"
 
 TestHash*
@@ -40,7 +40,7 @@ static void
 test_Equals(TestBatchRunner *runner) {
     Hash *hash  = Hash_new(0);
     Hash *other = Hash_new(0);
-    StackString *stuff = SSTR_WRAP_UTF8("stuff", 5);
+    String *stuff = SSTR_WRAP_C("stuff");
 
     TEST_TRUE(runner, Hash_Equals(hash, (Obj*)other),
               "Empty hashes are equal");
@@ -65,18 +65,18 @@ static void
 test_Store_and_Fetch(TestBatchRunner *runner) {
     Hash          *hash         = Hash_new(100);
     Hash          *dupe         = Hash_new(100);
-    const uint32_t starting_cap = Hash_Get_Capacity(hash);
-    VArray        *expected     = VA_new(100);
-    VArray        *got          = VA_new(100);
-    StackString *twenty       = SSTR_WRAP_UTF8("20", 2);
-    StackString *forty        = SSTR_WRAP_UTF8("40", 2);
-    StackString *foo          = SSTR_WRAP_UTF8("foo", 3);
+    const size_t   starting_cap = Hash_Get_Capacity(hash);
+    Vector        *expected     = Vec_new(100);
+    Vector        *got          = Vec_new(100);
+    String        *twenty       = SSTR_WRAP_C("20");
+    String        *forty        = SSTR_WRAP_C("40");
+    String        *foo          = SSTR_WRAP_C("foo");
 
     for (int32_t i = 0; i < 100; i++) {
         String *str = Str_newf("%i32", i);
-        Hash_Store(hash, (Obj*)str, (Obj*)str);
-        Hash_Store(dupe, (Obj*)str, INCREF(str));
-        VA_Push(expected, INCREF(str));
+        Hash_Store(hash, str, (Obj*)str);
+        Hash_Store(dupe, str, INCREF(str));
+        Vec_Push(expected, INCREF(str));
     }
     TEST_TRUE(runner, Hash_Equals(hash, (Obj*)dupe), "Equals");
 
@@ -84,44 +84,52 @@ test_Store_and_Fetch(TestBatchRunner *runner) {
                 "Initial capacity sufficient (no rebuilds)");
 
     for (int32_t i = 0; i < 100; i++) {
-        Obj *key  = VA_Fetch(expected, i);
-        Obj *elem = Hash_Fetch(hash, key);
-        VA_Push(got, (Obj*)INCREF(elem));
+        String *key  = (String*)Vec_Fetch(expected, i);
+        Obj    *elem = Hash_Fetch(hash, key);
+        Vec_Push(got, (Obj*)INCREF(elem));
     }
 
-    TEST_TRUE(runner, VA_Equals(got, (Obj*)expected),
+    TEST_TRUE(runner, Vec_Equals(got, (Obj*)expected),
               "basic Store and Fetch");
     TEST_INT_EQ(runner, Hash_Get_Size(hash), 100,
                 "size incremented properly by Hash_Store");
 
-    TEST_TRUE(runner, Hash_Fetch(hash, (Obj*)foo) == NULL,
+    TEST_TRUE(runner, Hash_Fetch(hash, foo) == NULL,
               "Fetch against non-existent key returns NULL");
 
     Obj *stored_foo = INCREF(foo);
-    Hash_Store(hash, (Obj*)forty, stored_foo);
-    TEST_TRUE(runner, SStr_Equals(foo, Hash_Fetch(hash, (Obj*)forty)),
+    Hash_Store(hash, forty, stored_foo);
+    TEST_TRUE(runner, Str_Equals(foo, Hash_Fetch(hash, forty)),
               "Hash_Store replaces existing value");
     TEST_FALSE(runner, Hash_Equals(hash, (Obj*)dupe),
                "replacement value spoils equals");
     TEST_INT_EQ(runner, Hash_Get_Size(hash), 100,
                 "size unaffected after value replaced");
 
-    TEST_TRUE(runner, Hash_Delete(hash, (Obj*)forty) == stored_foo,
+    TEST_TRUE(runner, Hash_Delete(hash, forty) == stored_foo,
               "Delete returns value");
     DECREF(stored_foo);
     TEST_INT_EQ(runner, Hash_Get_Size(hash), 99,
                 "size decremented by successful Delete");
-    TEST_TRUE(runner, Hash_Delete(hash, (Obj*)forty) == NULL,
+    TEST_TRUE(runner, Hash_Delete(hash, forty) == NULL,
               "Delete returns NULL when key not found");
     TEST_INT_EQ(runner, Hash_Get_Size(hash), 99,
                 "size not decremented by unsuccessful Delete");
-    DECREF(Hash_Delete(dupe, (Obj*)forty));
-    TEST_TRUE(runner, VA_Equals(got, (Obj*)expected), "Equals after Delete");
+    DECREF(Hash_Delete(dupe, forty));
+    TEST_TRUE(runner, Vec_Equals(got, (Obj*)expected), "Equals after Delete");
 
     Hash_Clear(hash);
-    TEST_TRUE(runner, Hash_Fetch(hash, (Obj*)twenty) == NULL, "Clear");
+    TEST_TRUE(runner, Hash_Fetch(hash, twenty) == NULL, "Clear");
     TEST_TRUE(runner, Hash_Get_Size(hash) == 0, "size is 0 after Clear");
 
+    Hash_Clear(hash);
+    Hash_Store(hash, forty, NULL);
+    TEST_TRUE(runner, Hash_Fetch(hash, forty) == NULL, "Store NULL");
+    TEST_TRUE(runner, Hash_Get_Size(hash) == 1, "Size after Store NULL");
+    TEST_TRUE(runner, Hash_Delete(hash, forty) == NULL, "Delete NULL value");
+    TEST_TRUE(runner, Hash_Get_Size(hash) == 0,
+              "Size after Deleting NULL val");
+
     DECREF(hash);
     DECREF(dupe);
     DECREF(got);
@@ -129,52 +137,35 @@ test_Store_and_Fetch(TestBatchRunner *runner) {
 }
 
 static void
-test_Keys_Values_Iter(TestBatchRunner *runner) {
+test_Keys_Values(TestBatchRunner *runner) {
     Hash     *hash     = Hash_new(0); // trigger multiple rebuilds.
-    VArray   *expected = VA_new(100);
-    VArray   *keys;
-    VArray   *values;
+    Vector   *expected = Vec_new(100);
+    Vector   *keys;
+    Vector   *values;
 
     for (uint32_t i = 0; i < 500; i++) {
         String *str = Str_newf("%u32", i);
-        Hash_Store(hash, (Obj*)str, (Obj*)str);
-        VA_Push(expected, INCREF(str));
+        Hash_Store(hash, str, (Obj*)str);
+        Vec_Push(expected, INCREF(str));
     }
 
-    VA_Sort(expected, NULL, NULL);
+    Vec_Sort(expected);
 
     keys   = Hash_Keys(hash);
     values = Hash_Values(hash);
-    VA_Sort(keys, NULL, NULL);
-    VA_Sort(values, NULL, NULL);
-    TEST_TRUE(runner, VA_Equals(keys, (Obj*)expected), "Keys");
-    TEST_TRUE(runner, VA_Equals(values, (Obj*)expected), "Values");
-    VA_Clear(keys);
-    VA_Clear(values);
+    Vec_Sort(keys);
+    Vec_Sort(values);
+    TEST_TRUE(runner, Vec_Equals(keys, (Obj*)expected), "Keys");
+    TEST_TRUE(runner, Vec_Equals(values, (Obj*)expected), "Values");
+    Vec_Clear(keys);
+    Vec_Clear(values);
 
     {
-        Obj *key;
-        Obj *value;
-        Hash_Iterate(hash);
-        while (Hash_Next(hash, &key, &value)) {
-            VA_Push(keys, INCREF(key));
-            VA_Push(values, INCREF(value));
-        }
-    }
-
-    VA_Sort(keys, NULL, NULL);
-    VA_Sort(values, NULL, NULL);
-    TEST_TRUE(runner, VA_Equals(keys, (Obj*)expected), "Keys from Iter");
-    TEST_TRUE(runner, VA_Equals(values, (Obj*)expected), "Values from Iter");
-
-    {
-        StackString *forty = SSTR_WRAP_UTF8("40", 2);
-        StackString *nope  = SSTR_WRAP_UTF8("nope", 4);
-        Obj *key = Hash_Find_Key(hash, (Obj*)forty, SStr_Hash_Sum(forty));
-        TEST_TRUE(runner, Obj_Equals(key, (Obj*)forty), "Find_Key");
-        key = Hash_Find_Key(hash, (Obj*)nope, SStr_Hash_Sum(nope)),
-        TEST_TRUE(runner, key == NULL,
-                  "Find_Key returns NULL for non-existent key");
+        String *forty = SSTR_WRAP_C("40");
+        String *nope  = SSTR_WRAP_C("nope");
+        TEST_TRUE(runner, Hash_Has_Key(hash, forty), "Has_Key");
+        TEST_FALSE(runner, Hash_Has_Key(hash, nope),
+                   "Has_Key returns false for non-existent key");
     }
 
     DECREF(hash);
@@ -186,34 +177,34 @@ test_Keys_Values_Iter(TestBatchRunner *runner) {
 static void
 test_stress(TestBatchRunner *runner) {
     Hash     *hash     = Hash_new(0); // trigger multiple rebuilds.
-    VArray   *expected = VA_new(1000);
-    VArray   *keys;
-    VArray   *values;
+    Vector   *expected = Vec_new(1000);
+    Vector   *keys;
+    Vector   *values;
 
     for (uint32_t i = 0; i < 1000; i++) {
         String *str = TestUtils_random_string(rand() % 1200);
-        while (Hash_Fetch(hash, (Obj*)str)) {
+        while (Hash_Fetch(hash, str)) {
             DECREF(str);
             str = TestUtils_random_string(rand() % 1200);
         }
-        Hash_Store(hash, (Obj*)str, (Obj*)str);
-        VA_Push(expected, INCREF(str));
+        Hash_Store(hash, str, (Obj*)str);
+        Vec_Push(expected, INCREF(str));
     }
 
-    VA_Sort(expected, NULL, NULL);
+    Vec_Sort(expected);
 
     // Overwrite for good measure.
     for (uint32_t i = 0; i < 1000; i++) {
-        String *str = (String*)VA_Fetch(expected, i);
-        Hash_Store(hash, (Obj*)str, INCREF(str));
+        String *str = (String*)Vec_Fetch(expected, i);
+        Hash_Store(hash, str, INCREF(str));
     }
 
     keys   = Hash_Keys(hash);
     values = Hash_Values(hash);
-    VA_Sort(keys, NULL, NULL);
-    VA_Sort(values, NULL, NULL);
-    TEST_TRUE(runner, VA_Equals(keys, (Obj*)expected), "stress Keys");
-    TEST_TRUE(runner, VA_Equals(values, (Obj*)expected), "stress Values");
+    Vec_Sort(keys);
+    Vec_Sort(values);
+    TEST_TRUE(runner, Vec_Equals(keys, (Obj*)expected), "stress Keys");
+    TEST_TRUE(runner, Vec_Equals(values, (Obj*)expected), "stress Values");
 
     DECREF(keys);
     DECREF(values);
@@ -224,10 +215,10 @@ test_stress(TestBatchRunner *runner) {
 static void
 test_store_skips_tombstone(TestBatchRunner *runner) {
     Hash *hash = Hash_new(0);
-    uint32_t mask = Hash_Get_Capacity(hash) - 1;
+    size_t mask = Hash_Get_Capacity(hash) - 1;
 
     String *one = Str_newf("one");
-    uint32_t slot = Str_Hash_Sum(one) & mask;
+    size_t slot = Str_Hash_Sum(one) & mask;
 
     // Find a colliding key.
     String *two = NULL;
@@ -240,10 +231,10 @@ test_store_skips_tombstone(TestBatchRunner *runner) {
         two = NULL;
     }
 
-    Hash_Store(hash, (Obj*)one, (Obj*)CFISH_TRUE);
-    Hash_Store(hash, (Obj*)two, (Obj*)CFISH_TRUE);
-    Hash_Delete(hash, (Obj*)one);
-    Hash_Store(hash, (Obj*)two, (Obj*)CFISH_TRUE);
+    Hash_Store(hash, one, (Obj*)CFISH_TRUE);
+    Hash_Store(hash, two, (Obj*)CFISH_TRUE);
+    Hash_Delete(hash, one);
+    Hash_Store(hash, two, (Obj*)CFISH_TRUE);
 
     TEST_INT_EQ(runner, Hash_Get_Size(hash), 1, "Store skips tombstone");
 
@@ -254,11 +245,11 @@ test_store_skips_tombstone(TestBatchRunner *runner) {
 
 void
 TestHash_Run_IMP(TestHash *self, TestBatchRunner *runner) {
-    TestBatchRunner_Plan(runner, (TestBatch*)self, 28);
+    TestBatchRunner_Plan(runner, (TestBatch*)self, 30);
     srand((unsigned int)time((time_t*)NULL));
     test_Equals(runner);
     test_Store_and_Fetch(runner);
-    test_Keys_Values_Iter(runner);
+    test_Keys_Values(runner);
     test_stress(runner);
     test_store_skips_tombstone(runner);
 }
